/**********************************************************************
 
	Copyright (C) 2005- Hirohisa MORI <joshua@nichibun.ac.jp>
 
	This program is free software; you can redistribute it 
	and/or modify it under the terms of the GLOBALBASE 
	Library General Public License (G-LGPL) as published by 

	http://www.globalbase.org/
 
	This program is distributed in the hope that it will be 
	useful, but WITHOUT ANY WARRANTY; without even the 
	implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE.

**********************************************************************/

#include	"machine/include.h"
#include	"memory_debug.h"
#include	"pri_level.h"
#include	"lock_level.h"
#include	"utils.h"
#include	"task.h"
#include	"matrix.h"
#include	"xl.h"
#include	"xlerror.h"


XL_SEXP *
xl_mxSave(XLISP_ENV * env,XL_SEXP * s,XLISP_ENV * a,XL_SYM_FIELD * sf);

void
init_mxSave(XLISP_ENV * env)
{
	set_env(env,l_string(std_cm,"mxSave"),
		get_func_prim(xl_mxSave,FO_APPLICATIVE,0,1,1));
}


XL_SEXP *
xl_mxSave(XLISP_ENV * env,XL_SEXP * s,XLISP_ENV * a,XL_SYM_FIELD * sf)
{
MATRIX_TOKEN * t;
int ret;
L_CHAR * option;

	t = get_env_work(env);
	if ( t == 0 )
		return 0;
	if ( t->process_node->status != MS_OK )
		goto matrix_err;
	option = get_sf_attribute(sf,l_string(std_cm,"option"));
	if ( option == 0 || l_strcmp(option,l_string(std_cm,"save")) == 0 ) {
		if ( t->process_node->matrix->p.write_file ) {
			lock_save(t->process_node);
/*
ss_printf("SAVE %lli %lli %lli\n",
t->process_node->dim_code[0],
t->process_node->dim_code[1],
t->process_node->dim_code[2]);
*/
			if ( t->process_node->status == MS_OK )
				ret = (*t->process_node->matrix->p.write_file)(t);
			else	ret = ME_ERR_NODE;
//ss_printf("END\n");
		}
		else 	goto no_suppport;
		if ( ret < 0 )
			goto matrix_err;
		if ( ret > 0 )
			return matrix_error("mxSave",s,AME_INTERRUPT,0);
		return 0;
	}
	else if ( l_strcmp(option,l_string(std_cm,"clear")) == 0 ) {
		lock_save(t->process_node);
		unlock_save(t->process_node);
		return 0;
	}
	else {
		return get_error(
			s->h.file,
			s->h.line,
			XLE_PROTO_INV_PARAM,
			l_string(std_cm,"mxSave"),
			List(n_get_string("invalid option"),
				get_string(option),
				-1));
	}
no_suppport:
	return get_error(
		s->h.file,
		s->h.line,
		XLE_PROTO_UNSUPPORT_PROTO,
		l_string(std_cm,"mxSave"),
		List(n_get_string("not support write_file operation"),
			get_integer((INTEGER64)(int)
				t->process_node->matrix->p.read_file,0),
			get_integer((INTEGER64)(int)
				t->process_node->matrix->p.read_net,0),
			get_integer((INTEGER64)(int)
				t->process_node->matrix->p.write_file,0),
			-1));
matrix_err:
	return get_error(
		s->h.file,
		s->h.line,
		XLE_PROTO_INV_OBJECT,
		l_string(std_cm,"mxSave"),
		List(n_get_string("invalid matrix saving"),
			get_integer(ret,0),
			-1));
}





